/* $Id: in_mp4.c,v 1.4 2005/06/28 19:19:31 bedlam Exp $ */
/* in_mp4.c
 * Plugin to read MP4 files as PCM
 *
 * Copyright (c) 2004 Brendan Cully <brendan@xiph.org>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 * $Id: in_mp4.c,v 1.4 2005/06/28 19:19:31 bedlam Exp $
 */

#include "definitions.h"
#include "in_mp4.h"
#include "metadata.h"

#include <string.h>

/* fink error? */
#define HAVE_IN_PORT_T 1
#define HAVE_SOCKLEN_T 1
#include <mp4.h>
#include <faad.h>

/* -- data structures -- */
typedef struct {
  MP4FileHandle mp4file;
  MP4TrackId track;
  faacDecHandle decoder;
  MP4SampleId cur_sample;
} mp4_in_t;

/* -- static prototypes -- */
static void stream404_mp4_read_metadata (MP4FileHandle mp4file);
static int stream404_mp4_readpcm (input_stream_t* self, size_t len,
				int16_t* left, int16_t* right);
static int stream404_mp4_close (input_stream_t* self);

/* try to open an MP4 file for decoding. Returns:
 *   0: success
 *   1: not an MP4 file
 *  -1: error opening
 */
int
stream404_mp4_open (input_stream_t* self, char* buf, size_t len)
{
  mp4_in_t* mp4_data;
  MP4FileHandle mp4file;
  MP4TrackId track;
  faacDecHandle decoder;
  unsigned int tracks;
  unsigned int i;
  unsigned char *escfg;
  unsigned int escfglen;
  unsigned int bitrate;
  unsigned long samplerate;
  unsigned char channels;
  
  /* At the moment we can only open seekable streams */
  if (! self->filesize)
    return 1;

  if ((mp4file = MP4Read(self->path, 0)) != MP4_INVALID_FILE_HANDLE) {
    stream404_mp4_read_metadata(mp4file);
  } else
    return 1;

  /* find audio stream */
  track = MP4_INVALID_TRACK_ID;
  tracks = MP4GetNumberOfTracks(mp4file, MP4_AUDIO_TRACK_TYPE, 0);
  for (i = 0; i < tracks; i++) {
    if ((track = MP4FindTrackId(mp4file, i, MP4_AUDIO_TRACK_TYPE, 0)) != MP4_INVALID_TRACK_ID)
      break;
  }
  if (track == MP4_INVALID_TRACK_ID) {
    stream404_log_error("stream404_mp4_open: No audio track found");
    goto errMP4;
  }

  MP4GetTrackESConfiguration(mp4file, track, &escfg, &escfglen);
  if (!escfg) {
    stream404_log_error("stream404_mp4_open: No audio format information found");
    goto errMP4;
  }

  bitrate = MP4GetTrackBitRate(mp4file, track)/1000;

  if (!(decoder = faacDecOpen())) {
    stream404_log_error("stream404_mp4_open: Could not get a FAAD handle");
    goto errMP4;
  }

  if (faacDecInit2(decoder, escfg, escfglen, &samplerate, &channels) < 0) {
    stream404_log_error("stream404_mp4_open: Could not initialise FAAD");
    free(escfg);
    goto errFAAC;
  }
  free(escfg);

  stream404_log_debug("Found MP4 audio at track %u, avg bitrate %u kbps, sample rate %ui hz, %u channels", track, bitrate, samplerate, channels);

  if (channels != 2) {
    stream404_log_error("stream404_mp4_open: Bad number of channels");
    goto errFAAC;
  }

  if (!(mp4_data = (mp4_in_t*)malloc (sizeof (mp4_in_t)))) {
    stream404_log_error ("Malloc failed in stream404_mp4_open");
    goto errFAAC;
  }

  close(self->fd);

  self->bitrate = bitrate;
  self->samplerate = samplerate;
  self->channels = channels;

  mp4_data->mp4file = mp4file;
  mp4_data->track = track;
  mp4_data->decoder = decoder;
  mp4_data->cur_sample = 1;

  self->type = STREAM404_INPUT_MP4;
  self->data = mp4_data;

  self->read = NULL;
  self->readpcm = stream404_mp4_readpcm;
  self->close = stream404_mp4_close;

  return 0;

errFAAC:
  faacDecClose(decoder);
errMP4:
  MP4Close(mp4file);

  return -1;
}

static void 
stream404_mp4_read_metadata (MP4FileHandle mp4file)
{
  char *value;
  char *artist = NULL, *title = NULL;
  uint16_t numvalue, numvalue2;
  if (MP4GetMetadataName(mp4file, &value) && value != NULL) {
    stream404_log_debug("Title: %s", value);
    title = stream404_util_strdup(value);
    stream404_util_free(value);
  }
  if (MP4GetMetadataArtist(mp4file, &value) && value != NULL) {
    stream404_log_debug("Artist: %s", value);
    artist = stream404_util_strdup(value);
    stream404_util_free(value);
  }
  if (MP4GetMetadataWriter(mp4file, &value) && value != NULL) {
    stream404_log_debug("Writer: %s", value);
    stream404_util_free(value);
  }
  if (MP4GetMetadataComment(mp4file, &value) && value != NULL) {
    stream404_log_debug("Comment: %s", value);
    stream404_util_free(value);
  }
  if (MP4GetMetadataTool(mp4file, &value) && value != NULL) {
    stream404_log_debug("Tool: %s", value);
    stream404_util_free(value);
  }
  if (MP4GetMetadataYear(mp4file, &value) && value != NULL) {
    stream404_log_debug("Year: %s", value);
    stream404_util_free(value);
  }
  if (MP4GetMetadataAlbum(mp4file, &value) && value != NULL) {
    stream404_log_debug("Album: %s", value);
    stream404_util_free(value);
  }
  if (MP4GetMetadataTrack(mp4file, &numvalue, &numvalue2)) {
    stream404_log_debug("Track: %u of %u", numvalue,
			numvalue2);
  }
  if (MP4GetMetadataDisk(mp4file, &numvalue, &numvalue2)) {
    stream404_log_debug("Disk: %u of %u", numvalue,
			numvalue2);
  }
  if (MP4GetMetadataGenre(mp4file, &value) && value != NULL) {
    stream404_log_debug("Genre: %s", value);
    stream404_util_free(value);
  }
  if (MP4GetMetadataTempo(mp4file, &numvalue)) {
    stream404_log_debug("Tempo: %u", numvalue);
  }
  stream404_metadata_set(artist, title);
  if (artist)
    stream404_util_free(artist);
  if (title)
    stream404_util_free(title);

  return;
}

static int
stream404_mp4_readpcm (input_stream_t* self, size_t olen, int16_t* left,
		     int16_t* right)
{
  mp4_in_t* mp4_data = (mp4_in_t*)self->data;
  unsigned char* buf;
  unsigned int blen;
  faacDecFrameInfo fi;
  void* decbuf;
  int i;

  fi.samples = 0;
  while (fi.samples == 0) {
    buf = NULL;
    if (!MP4ReadSample(mp4_data->mp4file, mp4_data->track, mp4_data->cur_sample++,
		       &buf, &blen, NULL, NULL, NULL, NULL) || !blen)
    {
      stream404_log_error("Error reading MP4");
      return 0;
    }

    self->bytes_read += blen;
    
    decbuf = faacDecDecode(mp4_data->decoder, &fi, buf, blen);
    free (buf);
    if (fi.error) {
      stream404_log_error("Error decoding MP4: %s", faacDecGetErrorMessage(fi.error));
      return 0;
    }

  }
  i = 0;
  while (i < fi.samples * 2) {
    *left++ = ((int16_t*)decbuf)[i++];
    *right++ = ((int16_t*)decbuf)[i++];
  }
  
  return fi.samples / 2;
}

static int
stream404_mp4_close (input_stream_t* self)
{
  mp4_in_t* mp4_data = (mp4_in_t*) self->data;

  faacDecClose(mp4_data->decoder);
  MP4Close(mp4_data->mp4file);
  free (mp4_data);

  return 0;
}
