/* $Id: playlist.c,v 1.4 2005/07/18 18:10:47 bedlam Exp $ */
/* playlist.c
 * - Functions for playlist handling
 * Copyright (c) 2000 Alexander Havng
 * Copyright (c) 2001-2 Brendan Cully
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */

#include "definitions.h"
#include "playlist.h"

extern stream404_config_t stream404_config;
static int playlist_init = 0;

#define MAIN 0
#define BACKUP 1

/* Global function definitions */

/* Wrapper function for the specific playlist handler's current line number.
 * This might not be available if your playlist is a database or something
 * weird, but it's just for the cue file so it doesn't matter much */
int
stream404_playlist_get_current_lineno (void)
{
  if (stream404_config.pm.get_lineno)
     return stream404_config.pm.get_lineno ();

  return 0;
}

/* Wrapper for the playlist handler's next file function.
 * Remember that if this returns non-NULL then the return
 * value is free()ed by the caller. */
char *
stream404_playlist_get_next (void *arg)
{
  char *ret;
  if ((ret = stream404_config.pm.get_next (arg)) == NULL) {
    stream404_log ("Cannot get next file on main playlist module");
    /* problem with current playlist module, try the backup one if defined */
    if (stream404_config.pm_backup.get_next) {
      return stream404_config.pm_backup.get_next (arg);
    } 
  }
  return ret;
}

/* Wrapper for the playlist handler's next file function.
 * Remember that if this returns non-NULL then the return
 * value is free()ed by the caller. */
char *
stream404_playlist_get_next_urg (void *arg)
{
  char *ret;

  if (!stream404_config.pm.get_next_urg) {
    /* this function may not have been defined so call the basic one */
    return stream404_playlist_get_next(arg);
  }

  if ((ret = stream404_config.pm.get_next_urg (arg)) == NULL) {
    /* problem with current playlist module, try the backup one if defined */
    if (stream404_config.pm_backup.get_next_urg) {
      return stream404_config.pm_backup.get_next_urg (arg);
    } 
  }
  return ret;
}

/* Allows a script to override file metadata if it wants. Returns NULL
 *   to mean 'do it yourself'. Modules need not implement this. */
char*
stream404_playlist_get_metadata (void)
{
  if (stream404_config.pm.get_metadata)
    return stream404_config.pm.get_metadata ();

  return NULL;
}

/* Initialize the toplevel playlist handler */
int
stream404_playlist_initialize (void)
{
  int rc = -1;
  int rc2 = -1;

  stream404_log_debug ("Initializing playlist handler...");

  stream404_config.pm.reload = NULL;

  switch (stream404_config.pm.playlist_type) {
  case stream404_playlist_sql_e:
    rc = stream404_playlist_sql_initialize (&stream404_config.pm);
    break;
  case stream404_playlist_builtin_e:
    rc = stream404_playlist_builtin_initialize (&stream404_config.pm, MAIN);
    break;
  case stream404_playlist_python_e:
#ifdef HAVE_LIBPYTHON
    rc = stream404_playlist_python_initialize (&stream404_config.pm);
#else
    stream404_log_error ("This binary has no support for embedded python");
#endif
    break;
  case stream404_playlist_perl_e: 
#ifdef HAVE_LIBPERL
    rc = stream404_playlist_perl_initialize (&stream404_config.pm);
#else
    stream404_log_error ("This binary has no support for embedded perl");
#endif
    break;
  default:
    stream404_log_error ("Unknown playlist module!");
    break;
  }

  if (rc < 0) {
    stream404_log ("Initialization of playlist handler failed. [%s]", stream404_log_get_error ());
    stream404_setup_shutdown ();
  }

  /* after having initialized master playlist module, do the same for the backup one */
  if (stream404_config.pm_backup.playlist_type != stream404_playlist_null_e) {
    stream404_log_debug ("Initializing backup playlist handler...");
    
    stream404_config.pm_backup.reload = NULL;
    
    switch (stream404_config.pm_backup.playlist_type) {
    case stream404_playlist_sql_e:
      rc2 = stream404_playlist_sql_initialize (&stream404_config.pm_backup);
      break;
    case stream404_playlist_builtin_e:
      rc2 = stream404_playlist_builtin_initialize (&stream404_config.pm_backup,
						   BACKUP);
      break;
    case stream404_playlist_python_e:
#ifdef HAVE_LIBPYTHON
      rc2 = stream404_playlist_python_initialize (&stream404_config.pm_backup);
#else
      stream404_log_error ("This binary has no support for embedded python");
#endif
      break;
    case stream404_playlist_perl_e: 
#ifdef HAVE_LIBPERL
      rc2 = stream404_playlist_perl_initialize (&stream404_config.pm_backup);
#else
      stream404_log_error ("This binary has no support for embedded perl");
#endif
      break;
    default:
      stream404_log_error ("Unknown backup playlist module!");
      break;
    }
    
    if (rc2 < 0) {
      stream404_log ("Initialization of backup playlist handler failed. [%s]", stream404_log_get_error ());
      stream404_config.pm_backup.get_next = NULL;
      stream404_config.pm_backup.get_next_urg = NULL;
      stream404_config.pm_backup.get_metadata = NULL;
      stream404_config.pm_backup.get_lineno = NULL;
      stream404_config.pm_backup.shutdown = NULL;
    }
  }

  playlist_init = 1;
  return rc;
}

/* Reload the playlist module */
int
stream404_playlist_reload (void)
{
  if (stream404_config.pm.reload)
    return stream404_config.pm.reload ();

  return 0;
}

/* Shutdown the playlist handler */
void
stream404_playlist_shutdown (void)
{
  if (playlist_init) {
    stream404_config.pm.shutdown ();
    if (stream404_config.pm_backup.shutdown) {
      stream404_config.pm_backup.shutdown ();
    }
  }
}
